From e93291399e7f60613d41a9b98fb8240dcb3a6853 Mon Sep 17 00:00:00 2001
From: Wu Jianhua <jianhua.wu@intel.com>
Date: Wed, 23 Feb 2022 00:27:33 +0800
Subject: [PATCH 01/10] avfilter/vf_blend_vulkan: add addition blend mode

Use the commands below to test: (href: https://trac.ffmpeg.org/wiki/Blend)

I. make an image for test
ffmpeg -f lavfi -i color=s=256x256,geq=r='H-1-Y':g='H-1-Y':b='H-1-Y' -frames 1 \
-y -pix_fmt yuv420p test.jpg

II. blend in sw
ffmpeg -i test.jpg -vf "split[a][b];[b]transpose[b];[a][b]blend=all_mode=addition,\
pseudocolor=preset=turbo" -y addition_sw.jpg

III. blend in vulkan
ffmpeg -init_hw_device vulkan -i test.jpg -vf "split[a][b];[b]transpose[b];\
[a]hwupload[a];[b]hwupload[b];[a][b]blend_vulkan=all_mode=addition,hwdownload,\
format=yuv420p,pseudocolor=preset=turbo" -y addition_vulkan.jpg

Signed-off-by: Wu Jianhua <jianhua.wu@intel.com>
---
 libavfilter/vf_blend_vulkan.c | 9 +++++++++
 1 file changed, 9 insertions(+)

diff --git a/libavfilter/vf_blend_vulkan.c b/libavfilter/vf_blend_vulkan.c
index fcc21cbc8d..b0349512a8 100644
--- a/libavfilter/vf_blend_vulkan.c
+++ b/libavfilter/vf_blend_vulkan.c
@@ -68,9 +68,16 @@ static const char blend_##MODE##_func[] = { \
 #define A top
 #define B bottom
 
+#define MAX vec4(1.0)
+
+#ifdef min
+#undef min
+#endif
+
 #define FN(EXPR) A + ((EXPR) - A) * opacity
 
 DEFINE_BLEND_MODE(NORMAL, A * opacity + B * (1.0f - opacity))
+DEFINE_BLEND_MODE(ADDITION, FN(min(MAX, A + B)))
 DEFINE_BLEND_MODE(MULTIPLY, FN(1.0f * A * B / 1.0f))
 
 static inline void init_blend_func(FilterParamsVulkan *param)
@@ -81,6 +88,7 @@ static inline void init_blend_func(FilterParamsVulkan *param)
             break;
 
     switch (param->mode) {
+    CASE(ADDITION)
     CASE(NORMAL)
     CASE(MULTIPLY)
     default: param->blend = NULL; break;
@@ -452,6 +460,7 @@ static const AVOption blend_vulkan_options[] = {
     { "c2_mode", "set component #2 blend mode", OFFSET(params[2].mode), AV_OPT_TYPE_INT, { .i64 = 0 }, 0, BLEND_NB - 1, FLAGS, "mode" },
     { "c3_mode", "set component #3 blend mode", OFFSET(params[3].mode), AV_OPT_TYPE_INT, { .i64 = 0 }, 0, BLEND_NB - 1, FLAGS, "mode" },
     { "all_mode", "set blend mode for all components", OFFSET(all_mode), AV_OPT_TYPE_INT, { .i64 = -1 }, -1, BLEND_NB - 1, FLAGS, "mode" },
+        { "addition", "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_ADDITION }, 0, 0, FLAGS, "mode" },
         { "normal",   "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_NORMAL   }, 0, 0, FLAGS, "mode" },
         { "multiply", "", 0, AV_OPT_TYPE_CONST, { .i64 = BLEND_MULTIPLY }, 0, 0, FLAGS, "mode" },
 
-- 
2.25.1

